#!/usr/bin/env python3
"""Setup script for backwards compatibility.

Modern package configuration is in pyproject.toml.
This file is provided for compatibility with older pip versions.
"""

from setuptools import setup

# Read the pyproject.toml for configuration
# All configuration is in pyproject.toml
setup()
